#include <stdio.h>
#include <math.h>
#include <windows.h>

struct Item {
	int key;
	int value;
};

extern "C" void sortRoutine(Item *items, int count);

#define STR2(X) #X
#define STR(X) STR2(X)

int main()
{
	int itemCount = 1000000;
	Item *items = new Item[itemCount];

	LARGE_INTEGER start, end, freq;

	QueryPerformanceFrequency(&freq);

	int numRuns = 100;
	int best = 99999;
	for (int run=0;run<numRuns;run++)
	{
		srand(12345);
		for (int n=0;n<itemCount;n++)
		{
			items[n].key = rand();
			items[n].value = rand();
		}

		QueryPerformanceCounter(&start);
		sortRoutine(items, itemCount);
		QueryPerformanceCounter(&end);

		// check it worked
		int prev = -1;
		for (int n=0;n<itemCount;n++)
		{
			if (items[n].key < prev) {
				printf("ERROR: sort failed\n");
				return 1;
			}
			prev = items[n].key;
		}

		unsigned __int64 diff = end.QuadPart - start.QuadPart;
		int ms = (int)round(1000.0 * (double)diff / (double)freq.QuadPart);
		if (ms < best)
			best = ms;
	}

	printf("%-30s: Took %ims best-case across %i runs\n", STR(VERSION), best, numRuns);
	return 0;
}